"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.extract = extract;
exports.collect = collect;
exports.cleanObsolete = cleanObsolete;

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

var _chalk = _interopRequireDefault(require("chalk"));

var _ramda = _interopRequireDefault(require("ramda"));

var extractors = _interopRequireWildcard(require("./extractors"));

function extract(srcPaths, targetPath) {
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var _options$ignore = options.ignore,
      ignore = _options$ignore === void 0 ? [] : _options$ignore,
      _options$verbose = options.verbose,
      verbose = _options$verbose === void 0 ? false : _options$verbose;
  var ignorePattern = ignore.length ? new RegExp(ignore.join("|"), "i") : null;
  srcPaths.forEach(function (srcFilename) {
    if (!_fs.default.existsSync(srcFilename) || ignorePattern && ignorePattern.test(srcFilename)) return;

    if (_fs.default.lstatSync(srcFilename).isDirectory()) {
      var subdirs = _fs.default.readdirSync(srcFilename).map(function (filename) {
        return _path.default.join(srcFilename, filename);
      });

      extract(subdirs, targetPath, options);
      return;
    }

    var extracted = _ramda.default.values(extractors).some(function (ext) {
      if (!ext.match(srcFilename)) return false;
      ext.extract(srcFilename, targetPath);
      return true;
    });

    if (extracted && verbose) console.log(_chalk.default.green(srcFilename));
  });
}

function collect(buildDir) {
  return _fs.default.readdirSync(buildDir).map(function (filename) {
    var filepath = _path.default.join(buildDir, filename);

    if (_fs.default.lstatSync(filepath).isDirectory()) {
      return collect(filepath);
    }

    if (!filename.endsWith(".json")) return;

    try {
      return JSON.parse(_fs.default.readFileSync(filepath).toString());
    } catch (e) {
      return {};
    }
  }).filter(Boolean).reduce(function (catalog, messages) {
    var mergeMessage = function mergeMessage(msgId, prev, next) {
      return (0, _objectSpread2.default)({}, next, {
        origin: _ramda.default.concat(prev.origin, next.origin)
      });
    };

    return _ramda.default.mergeWithKey(mergeMessage, catalog, messages);
  }, {});
}

function cleanObsolete(catalogs) {
  return _ramda.default.map(_ramda.default.filter(function (message) {
    return !message.obsolete;
  }), catalogs);
}
