"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _stringify = _interopRequireDefault(require("@babel/runtime/core-js/json/stringify"));

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

var _mkdirp = _interopRequireDefault(require("mkdirp"));

var _glob = _interopRequireDefault(require("glob"));

var _ramda = _interopRequireDefault(require("ramda"));

var locales = _interopRequireWildcard(require("./utils/locales"));

var sourceFilename = _path.default.join("{locale}", "messages.json");

var compiledFilename = _path.default.join("{locale}", "messages.js");

var _default = function _default(config) {
  return {
    formatFilename: function formatFilename(pattern, locale) {
      return pattern.replace("{locale}", locale);
    },
    write: function write(locale, messages) {
      var filename = _path.default.join(config.localeDir, this.formatFilename(sourceFilename, locale));

      var created = !_fs.default.existsSync(filename);

      _fs.default.writeFileSync(filename, (0, _stringify.default)(messages, null, 2));

      return [created, filename];
    },
    read: function read(locale) {
      var filename = _path.default.join(config.localeDir, this.formatFilename(sourceFilename, locale));

      if (!_fs.default.existsSync(filename)) return null;

      var raw = _fs.default.readFileSync(filename).toString();

      try {
        return JSON.parse(raw);
      } catch (e) {
        return null;
      }
    },
    merge: function merge(nextCatalog) {
      var _this = this;

      var nextKeys = _ramda.default.keys(nextCatalog);

      return _ramda.default.mergeAll(this.getLocales().map(function (locale) {
        var prevCatalog = _this.read(locale);

        if (!prevCatalog) return nextCatalog;

        var prevKeys = _ramda.default.keys(prevCatalog);

        var newKeys = _ramda.default.difference(nextKeys, prevKeys);

        var mergeKeys = _ramda.default.intersection(nextKeys, prevKeys);

        var obsoleteKeys = _ramda.default.difference(prevKeys, nextKeys); // Initialize new catalog with new keys


        var newMessages = _ramda.default.mapObjIndexed(function (message, key) {
          return (0, _objectSpread2.default)({
            translation: config.sourceLocale === locale ? key : ""
          }, message);
        }, _ramda.default.pick(newKeys, nextCatalog)); // Merge translations from previous catalog


        var mergedMessages = mergeKeys.map(function (key) {
          return (0, _defineProperty2.default)({}, key, (0, _objectSpread2.default)({
            translation: prevCatalog[key].translation
          }, _ramda.default.omit(["obsolete, translation"], nextCatalog[key])));
        }); // Mark all remaining translations as obsolete

        var obsoleteMessages = obsoleteKeys.map(function (key) {
          return (0, _defineProperty2.default)({}, key, (0, _objectSpread2.default)({}, prevCatalog[key], {
            obsolete: true
          }));
        });

        var newCatalog = _ramda.default.mergeAll([newMessages].concat((0, _toConsumableArray2.default)(mergedMessages), (0, _toConsumableArray2.default)(obsoleteMessages)));

        return (0, _defineProperty2.default)({}, locale, newCatalog);
      }));
    },
    getTranslation: function getTranslation(catalogs, locale, key, _ref4) {
      var fallbackLocale = _ref4.fallbackLocale,
          sourceLocale = _ref4.sourceLocale;

      var getTranslation = function getTranslation(locale) {
        return catalogs[locale][key].translation;
      };

      return (// Get translation in target locale
        getTranslation(locale) || // Get translation in fallbackLocale (if any)
        fallbackLocale && getTranslation(fallbackLocale) || // Get message default
        catalogs[locale][key].defaults || // If sourceLocale is either target locale of fallback one, use key
        sourceLocale && sourceLocale === locale && key || sourceLocale && fallbackLocale && sourceLocale === fallbackLocale && key || // Otherwise no translation is available
        undefined
      );
    },
    writeCompiled: function writeCompiled(locale, content) {
      var filename = _path.default.join(config.localeDir, this.formatFilename(compiledFilename, locale));

      _fs.default.writeFileSync(filename, content);

      return filename;
    },
    getLocale: function getLocale(filename) {
      var filenameRe = new RegExp(this.formatFilename(sourceFilename, locales.localeRe.source));
      var match = filenameRe.exec(filename);
      if (!match) return null;
      return match[1];
    },
    getLocales: function getLocales() {
      var _this2 = this;

      var pattern = _path.default.join(config.localeDir, this.formatFilename(sourceFilename, "*"));

      return _glob.default.sync(pattern).map(function (filename) {
        return _this2.getLocale(filename);
      }).filter(Boolean);
    },
    addLocale: function addLocale(locale) {
      if (!locales.isValid(locale)) {
        return [false, null];
      }

      var filename = _path.default.join(config.localeDir, this.formatFilename(sourceFilename, locale));

      if (!_fs.default.existsSync(filename)) {
        var dirname = _path.default.dirname(filename);

        _mkdirp.default.sync(dirname);

        this.write(locale, {});
        return [true, filename];
      }

      return [false, filename];
    }
  };
};

exports.default = _default;
