"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _keys = _interopRequireDefault(require("@babel/runtime/core-js/object/keys"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _chalk = _interopRequireDefault(require("chalk"));

var _ramda = _interopRequireDefault(require("ramda"));

var _commander = _interopRequireDefault(require("commander"));

var _makePlural = _interopRequireDefault(require("make-plural"));

var _conf = require("@lingui/conf");

var _compile = require("./api/compile");

function command(config, format, options) {
  var locales = format.getLocales();

  if (!locales.length) {
    console.log("No locales defined!\n");
    console.log("(use \"".concat(_chalk.default.yellow("lingui add-locale <language>"), "\" to add one)"));
    return false;
  }

  var catalogs = _ramda.default.mergeAll(locales.map(function (locale) {
    return (0, _defineProperty2.default)({}, locale, format.read(locale));
  }));

  var noMessages = _ramda.default.compose(_ramda.default.all(_ramda.default.equals(true)), _ramda.default.values, _ramda.default.map(_ramda.default.isEmpty));

  if (noMessages(catalogs)) {
    console.log("Nothing to compile, message catalogs are empty!\n");
    console.log("(use \"".concat(_chalk.default.yellow("lingui extract"), "\" to extract messages from source files)"));
    return false;
  }

  console.log("Compiling message catalogs…");
  return locales.map(function (locale) {
    var _locale$split = locale.split("_"),
        _locale$split2 = (0, _slicedToArray2.default)(_locale$split, 1),
        language = _locale$split2[0];

    if (!_makePlural.default[language]) {
      console.log(_chalk.default.red("Error: Invalid locale ".concat(_chalk.default.bold(locale), " (missing plural rules)!")));
      console.log();
      return false;
    }

    var messages = _ramda.default.mergeAll((0, _keys.default)(catalogs[locale]).map(function (key) {
      return (0, _defineProperty2.default)({}, key, format.getTranslation(catalogs, locale, key, {
        fallbackLocale: config.fallbackLocale,
        sourceLocale: config.sourceLocale
      }));
    }));

    if (!options.allowEmpty && config.sourceLocale !== locale) {
      var missing = _ramda.default.keys(messages).filter(function (key) {
        return messages[key] === undefined;
      });

      if (missing.length) {
        console.log(_chalk.default.red("Error: Failed to compile catalog for locale ".concat(_chalk.default.bold(locale), "!")));

        if (options.verbose) {
          console.log(_chalk.default.red("Missing translations:"));
          missing.forEach(function (msgId) {
            return console.log(msgId);
          });
        } else {
          console.log(_chalk.default.red("Missing ".concat(missing.length, " translation(s)")));
        }

        console.log();
        return false;
      }
    }

    var compiledCatalog = (0, _compile.createCompiledCatalog)(locale, messages);
    var compiledPath = format.writeCompiled(locale, compiledCatalog);
    options.verbose && console.log(_chalk.default.green("".concat(locale, " \u21D2 ").concat(compiledPath)));
    return compiledPath;
  });
}

if (require.main === module) {
  _commander.default.description("Add compile message catalogs and add language data (plurals) to compiled bundle.").option("--strict", "Disable defaults for missing translations").option("--verbose", "Verbose output").option("--format <format>", "Format of message catalog").on("--help", function () {
    console.log("\n  Examples:\n");
    console.log("    # Compile translations and use defaults or message IDs for missing translations");
    console.log("    $ lingui compile");
    console.log("");
    console.log("    # Compile translations but fail when there're missing");
    console.log("    # translations (don't replace missing translations with");
    console.log("    # default messages or message IDs)");
    console.log("    $ lingui compile --strict");
  }).parse(process.argv);

  var config = (0, _conf.getConfig)();
  var formatName = _commander.default.format || config.format;

  var format = require("./api/formats/".concat(formatName)).default(config);

  var results = command(config, format, {
    verbose: _commander.default.verbose || false,
    allowEmpty: !_commander.default.strict
  });

  if (!results || results.some(function (res) {
    return !res;
  })) {
    process.exit(1);
  }

  console.log("Done!");
}
