"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = command;

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

var _mkdirp = _interopRequireDefault(require("mkdirp"));

var _chalk = _interopRequireDefault(require("chalk"));

var _commander = _interopRequireDefault(require("commander"));

var _conf = require("@lingui/conf");

var _extract = require("./api/extract");

var _stats = require("./api/stats");

function command(config, format, options) {
  var convertFormat = options.prevFormat || format;
  var locales = convertFormat.getLocales();

  if (!locales.length) {
    console.log("No locales defined!\n");
    console.log("(use \"".concat(_chalk.default.yellow("lingui add-locale <language>"), "\" to add one)"));
    return false;
  }

  var buildDir = _path.default.join(config.localeDir, "_build");

  if (!_fs.default.existsSync(buildDir)) {
    (0, _mkdirp.default)(buildDir);
  }

  console.log("Extracting messages from source files…");
  (0, _extract.extract)(config.srcPathDirs, config.localeDir, {
    ignore: config.srcPathIgnorePatterns,
    verbose: options.verbose
  });
  options.verbose && console.log();
  console.log("Collecting all messages…");
  var clean = options.clean ? _extract.cleanObsolete : function (id) {
    return id;
  };
  var catalog = (0, _extract.collect)(buildDir);
  var catalogs = clean(convertFormat.merge(catalog));
  options.verbose && console.log();
  console.log("Writing message catalogues…");
  locales.map(function (locale) {
    return format.write(locale, catalogs[locale]);
  }).forEach(function (_ref) {
    var _ref2 = (0, _slicedToArray2.default)(_ref, 2),
        created = _ref2[0],
        filename = _ref2[1];

    if (!filename || !options.verbose) return;

    if (created) {
      console.log(_chalk.default.green("Created ".concat(filename)));
    } else {
      console.log(_chalk.default.green("Updated ".concat(filename)));
    }
  });
  options.verbose && console.log();
  console.log("Messages extracted!\n");
  console.log("Catalog statistics:");
  (0, _stats.printStats)(config, catalogs);
  console.log();
  console.log("(use \"".concat(_chalk.default.yellow("lingui add-locale <language>"), "\" to add more locales)"));
  console.log("(use \"".concat(_chalk.default.yellow("lingui extract"), "\" to update catalogs with new messages)"));
  console.log("(use \"".concat(_chalk.default.yellow("lingui compile"), "\" to compile catalogs for production)"));
  return true;
}

if (require.main === module) {
  _commander.default.option("--clean", "Remove obsolete translations").option("--verbose", "Verbose output").option("--format <format>", "Format of message catalogs").option("--convert-from <format>", "Convert from previous format of message catalogs").parse(process.argv);

  var config = (0, _conf.getConfig)();
  var formatName = _commander.default.format || config.format;
  var prevFormatName = _commander.default.convertFrom;

  if (prevFormatName && formatName === prevFormatName) {
    console.log(_chalk.default.red("Trying to migrate message catalog to the same format"));
    console.log("Set ".concat(_chalk.default.bold("new"), " format in lingui configuration or using") + " --format option\nand ".concat(_chalk.default.bold("previous"), " format using") + " --convert-format option.");
    console.log();
    console.log("Example: Convert from lingui format to minimal");
    console.log(_chalk.default.yellow("lingui extract --format minimal --convert-from lingui"));
    process.exit(1);
  }

  var format = require("./api/formats/".concat(formatName)).default(config);

  var _prevFormat = prevFormatName ? require("./api/formats/".concat(prevFormatName)).default(config) : null;

  var result = command(config, format, {
    verbose: _commander.default.verbose || false,
    clean: _commander.default.clean || false,
    prevFormat: _prevFormat
  });
  if (!result) process.exit(1);
}
